"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleTable = void 0;
const constants_js_1 = require("../constants.js");
const type_guards_js_1 = require("../utils/type-guards.js");
const computeMaxValue = (topBottom, leftRight, trIndex, tdIndex, points) => {
    topBottom[trIndex] = topBottom[trIndex] || [0, 0];
    leftRight[tdIndex] = leftRight[tdIndex] || [0, 0];
    topBottom[trIndex] = [
        Math.max(topBottom[trIndex][0], points?.[constants_js_1.POS_TOP] || 0),
        Math.max(topBottom[trIndex][1], points?.[constants_js_1.POS_BOTTOM] || 0)
    ];
    leftRight[tdIndex] = [
        Math.max(leftRight[tdIndex][0], points?.[constants_js_1.POS_LEFT] || 0),
        Math.max(leftRight[tdIndex][1], points?.[constants_js_1.POS_RIGHT] || 0)
    ];
};
const handleTable = (item) => {
    if ((0, type_guards_js_1.isTable)(item)) {
        const bodyItem = item.table.body[0]?.[0];
        const tableItem = bodyItem && typeof bodyItem !== 'string' && 'table' in bodyItem ? bodyItem : null;
        if (!tableItem) {
            return item;
        }
        const collapseBorder = item?.[constants_js_1.META]?.[constants_js_1.STYLE]?.['border-collapse'] === 'collapse';
        const innerTable = tableItem.table;
        const paddingsLeftRight = [];
        const paddingsTopBottom = [];
        const borderTopBottom = [];
        const borderLeftRight = [];
        innerTable.body
            .forEach((row, trIndex) => {
            row.forEach((column, tdIndex) => {
                if (column[constants_js_1.META]?.[constants_js_1.PADDING]) {
                    computeMaxValue(paddingsTopBottom, paddingsLeftRight, trIndex, tdIndex, column[constants_js_1.META]?.[constants_js_1.PADDING]);
                }
                if (column[constants_js_1.META]?.[constants_js_1.BORDER]) {
                    computeMaxValue(borderTopBottom, borderLeftRight, trIndex, tdIndex, column[constants_js_1.META]?.[constants_js_1.BORDER]);
                }
                if (typeof column !== 'string') {
                    column.style = column.style || [];
                    const nodeName = column[constants_js_1.META]?.[constants_js_1.NODE]?.nodeName === 'TH' ? 'th' : 'td';
                    column.style.push(tdIndex % 2 === 0 ? nodeName + ':nth-child(even)' : nodeName + ':nth-child(odd)');
                    column.style.push(trIndex % 2 === 0 ? 'tr:nth-child(even)' : 'tr:nth-child(odd)');
                }
            });
        });
        // Padding fix
        innerTable.body
            .forEach((row, trIndex) => {
            row.forEach((column, tdIndex) => {
                if (!column[constants_js_1.META]?.[constants_js_1.PADDING]?.[constants_js_1.POS_LEFT] && typeof column !== 'string' && paddingsLeftRight[tdIndex]) {
                    column.margin = column.margin || [0, 0, 0, 0];
                    if (column.margin) {
                        column.margin[constants_js_1.POS_LEFT] = column.margin[constants_js_1.POS_LEFT] - paddingsLeftRight[tdIndex][0];
                    }
                }
                if (!column[constants_js_1.META]?.[constants_js_1.PADDING]?.[constants_js_1.POS_TOP] && typeof column !== 'string' && paddingsTopBottom[trIndex]) {
                    column.margin = column.margin || [0, 0, 0, 0];
                    if (column.margin) {
                        column.margin[constants_js_1.POS_TOP] = column.margin[constants_js_1.POS_TOP] - paddingsTopBottom[trIndex][0];
                    }
                }
            });
        });
        const tableLayout = {
            defaultBorder: false
        };
        const defaultPadding = 0.75;
        tableLayout.paddingTop = (i) => paddingsTopBottom[i]?.[0] ?? defaultPadding;
        tableLayout.paddingBottom = (i) => paddingsTopBottom[i]?.[1] ?? defaultPadding;
        tableLayout.paddingLeft = (i) => paddingsLeftRight[i]?.[0] ?? defaultPadding;
        tableLayout.paddingRight = (i) => paddingsLeftRight[i]?.[1] ?? defaultPadding;
        if (borderTopBottom.length) {
            tableLayout.hLineWidth = (i) => {
                const even = i % 2 === 0;
                if (collapseBorder) {
                    return borderTopBottom[i]?.[0] ?? borderTopBottom[i - 1]?.[1] ?? 0;
                }
                if (i === 0) {
                    return 0;
                }
                const index = even ? i - 1 : i;
                return borderTopBottom[index]?.[even ? 1 : 0] ?? 0;
            };
        }
        if (borderLeftRight.length) {
            tableLayout.vLineWidth = (i) => {
                const even = i % 2 === 0;
                if (collapseBorder) {
                    return borderLeftRight[i]?.[0] ?? borderLeftRight[i - 1]?.[1] ?? 0;
                }
                if (i === 0) {
                    return 0;
                }
                const index = even ? i - 1 : i;
                return borderLeftRight[index]?.[even ? 1 : 0] ?? 0;
            };
        }
        tableItem.layout = tableLayout;
        if (collapseBorder) {
            item.layout = {
                ...(typeof item.layout !== 'string' ? item.layout : {}),
                paddingLeft: () => 0,
                paddingRight: () => 0,
                paddingTop: () => 0,
                paddingBottom: () => 0
            };
        }
    }
    return item;
};
exports.handleTable = handleTable;
