"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseTemplate = exports.parse = exports.getPatterns = exports.simpleRender = void 0;
const constants_js_1 = require("./constants.js");
Object.defineProperty(exports, "getPatterns", { enumerable: true, get: function () { return constants_js_1.getPatterns; } });
const context_js_1 = require("./context.js");
const default_config_js_1 = require("./default-config.js");
const index_js_1 = require("./parser/index.js");
const header_footer_js_1 = require("./render/header-footer.js");
const simple_render_js_1 = require("./render/simple-render.js");
Object.defineProperty(exports, "simpleRender", { enumerable: true, get: function () { return simple_render_js_1.simpleRender; } });
const html_to_dom_js_1 = require("./utils/html-to-dom.js");
const parse = (input, _config = (0, default_config_js_1.defaultConfig)()) => {
    const root = typeof input === 'string' ? (0, html_to_dom_js_1.htmlToDom)(input) : input;
    if (root?.nodeName === 'TEMPLATE' || root?.nodeName === 'HTML' || root?.nodeName === '#document') {
        return (0, exports.parseTemplate)(root, _config);
    }
    const ctx = (0, context_js_1.createContext)(_config);
    if (root) {
        const styles = root.querySelectorAll('style');
        const pageStyles = ctx.config.parseCss(styles);
        ctx.setPageStyles(pageStyles);
    }
    const content = root !== null ? (0, index_js_1.parseChildren)(root, ctx) : [];
    return {
        info: {
            title: 'PDF Document - created by html2pdfmake',
            creator: 'html2pdfmake',
            producer: 'html2pdfmake',
        },
        content: content,
        images: ctx.images,
        patterns: (0, constants_js_1.getPatterns)(),
        pageSize: 'A4',
        pageOrientation: 'portrait',
        header: () => [],
        footer: () => [],
        pageMargins: [40, 60, 40, 60]
    };
};
exports.parse = parse;
const parseTemplate = (template, _config = (0, default_config_js_1.defaultConfig)()) => {
    const ctx = (0, context_js_1.createContext)(_config);
    const root = template.nodeName === 'TEMPLATE'
        ? ctx.config.document().importNode(template.content, true)
        : template.nodeName === '#document' ? template.documentElement : template;
    const headers = (0, header_footer_js_1.headerFooterContent)(root.querySelectorAll(':scope > header, body > header'), ctx);
    const footers = (0, header_footer_js_1.headerFooterContent)(root.querySelectorAll(':scope > footer, body > footer'), ctx);
    const styles = root.querySelectorAll('style');
    const pageStyles = ctx.config.parseCss(styles);
    ctx.setPageStyles(pageStyles);
    const main = root.querySelector('main');
    const title = root.querySelector('title');
    const author = root.querySelector('meta[name="author"]');
    const subject = root.querySelector('meta[name="description"]');
    const keywords = root.querySelector('meta[name="keywords"]');
    const pageSize = root.querySelector('meta[name="page-size"]');
    const pageOrientation = root.querySelector('meta[name="page-orientation"]');
    const pageMargins = root.querySelector('meta[name="page-margins"]');
    const margins = pageMargins?.getAttribute('content')?.split(',').filter(m => m).map(m => parseFloat(m)) || [40, 60, 40, 60];
    const info = {
        title: title?.textContent || 'PDF Document - created by html2pdfmake',
        author: author?.getAttribute('content') || 'html2pdfmake',
        subject: subject?.getAttribute('content') || 'Created by html2pdfmake',
        keywords: keywords?.getAttribute('content') || 'html2pdfmake',
        creator: 'html2pdfmake',
        producer: 'html2pdfmake',
    };
    const content = main !== null ? (0, index_js_1.parseChildren)(main, ctx) : [];
    return {
        header(currentPage, pageCount) {
            return (0, header_footer_js_1.headerFooter)(currentPage, pageCount, headers);
        },
        footer(currentPage, pageCount) {
            return (0, header_footer_js_1.headerFooter)(currentPage, pageCount, footers);
        },
        info,
        content: content,
        images: ctx.images,
        patterns: (0, constants_js_1.getPatterns)(),
        pageSize: pageSize?.getAttribute('content') || 'A4',
        pageOrientation: pageOrientation?.getAttribute('content') === 'landscape' ? 'landscape' : 'portrait',
        pageMargins: margins,
    };
};
exports.parseTemplate = parseTemplate;
