"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseText = void 0;
const constants_js_1 = require("../constants.js");
const parseText = (el) => {
    const text = el.textContent;
    if (text === null) {
        return null;
    }
    const keepNewLines = text.replace(/[^\S\r\n]+/, '');
    const trimmedText = text.replace(/\n|\t| +/g, ' ')
        .replace(/^ +/, '')
        .replace(/ +$/, '');
    //.trim() removes also &nbsp;
    const endWithNL = keepNewLines.at(-1) === '\n';
    const startWithNL = keepNewLines[0] === '\n';
    const startWithWhitespace = text[0] === ' ';
    const endWithWhitespace = text.at(-1) === ' ';
    return {
        text: trimmedText,
        [constants_js_1.META]: {
            [constants_js_1.START_WITH_NEWLINE]: startWithNL,
            [constants_js_1.END_WITH_NEWLINE]: endWithNL,
            [constants_js_1.IS_NEWLINE]: startWithNL && trimmedText.length === 0,
            [constants_js_1.START_WITH_WHITESPACE]: startWithWhitespace,
            [constants_js_1.END_WITH_WHITESPACE]: endWithWhitespace,
            [constants_js_1.IS_WHITESPACE]: startWithWhitespace && text.length === 1,
        },
    };
};
exports.parseText = parseText;
