"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.attrToProps = void 0;
const constants_js_1 = require("../constants.js");
const index_js_1 = require("../handler/index.js");
const unique_id_js_1 = require("../utils/unique-id.js");
const unit_js_1 = require("../utils/unit.js");
// TODO make customizable
const computeStyleClasses = (item, cascadeClass = []) => {
    const el = item[constants_js_1.META]?.[constants_js_1.NODE];
    if (!el || !('getAttribute' in el)) {
        return [];
    }
    const cssClass = el.getAttribute('class') || '';
    const cssClasses = [...new Set(// TODO use classList?
        cssClass.split(' ')
            .filter((value) => value)
            .map((value) => '.' + value.trim()))];
    const nodeName = el.nodeName.toLowerCase();
    const parentNodeName = el.parentNode ? el.parentNode.nodeName.toLowerCase() : null;
    const styleNames = [
        nodeName,
    ]
        .concat(cascadeClass.map(cssClass => cssClass + ' ' + nodeName))
        .concat(cssClasses)
        .concat(cssClasses.map(cssClass => nodeName + cssClass));
    if (cssClasses.length > 2) {
        styleNames.push(cssClasses.join('')); // .a.b.c
    }
    if (parentNodeName) {
        styleNames.push(parentNodeName + '>' + nodeName);
    }
    const uniqueId = (0, unique_id_js_1.getUniqueId)(item);
    styleNames.push(uniqueId); // Should be the last one
    // TODO improve sort by specificity
    return [...new Set((item.style || []).concat(styleNames))];
};
const attrToProps = (item, parentItem) => {
    const el = item[constants_js_1.META]?.[constants_js_1.NODE];
    if (!el || !('getAttribute' in el))
        return { [constants_js_1.META]: { [constants_js_1.STYLE]: {} } };
    const cssClasses = computeStyleClasses(item, parentItem?.[constants_js_1.META]?.[constants_js_1.CLASSES]);
    const nodeName = el.nodeName.toLowerCase();
    const props = {
        [constants_js_1.META]: {
            [constants_js_1.STYLE]: item[constants_js_1.META]?.[constants_js_1.STYLE] || {},
            [constants_js_1.CLASSES]: (parentItem?.[constants_js_1.META]?.[constants_js_1.CLASSES] || []).concat(cssClasses),
            ...(item[constants_js_1.META] || {})
        },
        style: cssClasses
    };
    for (let i = 0; i < el.attributes.length; i++) {
        const name = el.attributes[i].name;
        const value = el.getAttribute(name)?.trim() || null;
        if (value == null) {
            continue;
        }
        props[constants_js_1.META][constants_js_1.STYLE][name] = value;
        switch (name) {
            case 'rowspan':
                props.rowSpan = parseInt(value, 10);
                break;
            case 'colspan':
                props.colSpan = parseInt(value, 10);
                break;
            case 'value':
                if (nodeName === 'li') {
                    props.counter = parseInt(value, 10);
                }
                break;
            case 'start': // ol
                if (nodeName === 'ol') {
                    props.start = parseInt(value, 10);
                }
                break;
            case 'width':
                if ('image' in item) {
                    props.width = (0, unit_js_1.toUnit)(value);
                }
                break;
            case 'height':
                if ('image' in item) {
                    props.height = (0, unit_js_1.toUnit)(value);
                }
                break;
            case 'data-fit':
                if (value === 'true') {
                    const width = el.getAttribute('width');
                    const height = el.getAttribute('height');
                    if (width && height) {
                        props.fit = [(0, unit_js_1.toUnit)(width), (0, unit_js_1.toUnit)(height)];
                    }
                }
                break;
            case 'data-toc-item':
                if (value !== 'false') {
                    let toc = {};
                    if (value) {
                        try {
                            toc = JSON.parse(value);
                        }
                        catch (e) {
                            console.warn('Not valid JSON format.', value);
                        }
                    }
                    props[constants_js_1.META][constants_js_1.POST_HANDLER] = (item) => {
                        (0, index_js_1.addTocItem)(item, toc);
                        return item;
                    };
                }
                break;
            case 'data-pdfmake':
                if (value) {
                    try {
                        props[constants_js_1.META][constants_js_1.PDFMAKE] = JSON.parse(value);
                    }
                    catch (e) {
                        console.warn('Not valid JSON format.', value);
                    }
                }
                break;
            default:
        }
    }
    return props;
};
exports.attrToProps = attrToProps;
