"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.computeBorder = void 0;
const constants_js_1 = require("../constants.js");
const type_guards_js_1 = require("../utils/type-guards.js");
const unit_js_1 = require("../utils/unit.js");
const getBorderStyle = (value) => {
    const border = value.split(' ');
    const color = border[2] || 'black';
    const borderStyle = border[1] || 'solid';
    const width = (0, unit_js_1.toUnit)(border[0]);
    return { color, width, borderStyle };
};
const colorLineLayout = (props, l) => {
    const layout = typeof props.layout === 'string' ? {} : props.layout || {};
    const colorFn = l === 'h' ? layout.hLineColor : layout.vLineColor;
    const widthFn = l === 'h' ? layout.hLineWidth : layout.vLineWidth;
    const lineWidth = widthFn || (() => 0);
    const lineColor = typeof colorFn === 'function'
        ? colorFn
        : () => typeof colorFn === 'string' ? colorFn : 'black';
    return {
        layout,
        lineWidth,
        lineColor
    };
};
const computeBorder = (item, props, directive, value) => {
    const { color, width, borderStyle } = getBorderStyle(value);
    const table = (0, type_guards_js_1.isTable)(item);
    const setColumnBorder = (index) => {
        const borderWidth = props[constants_js_1.META][constants_js_1.BORDER] || [0, 0, 0, 0];
        props.border = item.border || props.border || [false, false, false, false];
        props.borderColor = item.borderColor || props.borderColor || ['black', 'black', 'black', 'black'];
        if (value === 'none') {
            props.border[index] = false;
        }
        else {
            props.border[index] = width > 0;
            props.borderColor[index] = color;
        }
        borderWidth[index] = width;
        props[constants_js_1.META][constants_js_1.BORDER] = borderWidth;
    };
    switch (directive) {
        case 'border':
            if (table) {
                props.layout = item.layout || props.layout || {};
                if (typeof props.layout === 'string') {
                    props.layout = {};
                }
                if (value === 'none') {
                    props.layout.hLineWidth = () => 0;
                    props.layout.vLineWidth = () => 0;
                    break;
                }
                props.layout.vLineColor = () => color;
                props.layout.hLineColor = () => color;
                props.layout.hLineWidth = (i, node) => (i === 0 || i === node.table.body.length) ? width : 0;
                props.layout.vLineWidth = (i, node) => (i === 0 || i === node.table.widths?.length) ? width : 0;
                if (borderStyle === 'dashed') {
                    props.layout.hLineStyle = () => ({ dash: { length: 2, space: 2 } });
                    props.layout.vLineStyle = () => ({ dash: { length: 2, space: 2 } });
                }
            }
            else {
                setColumnBorder(0);
                setColumnBorder(1);
                setColumnBorder(2);
                setColumnBorder(3);
            }
            break;
        case 'border-bottom':
            if (table) {
                props.layout = item.layout || props.layout || {};
                const { layout, lineWidth: hLineWidth, lineColor: hLineColor } = colorLineLayout(props, 'h');
                layout.hLineWidth = (i, node, columnIndex) => (i === node.table.body.length) ? width : hLineWidth(i, node, columnIndex);
                layout.hLineColor = (i, node, columnIndex) => (i === node.table.body.length) ? color : hLineColor(i, node, columnIndex);
                props.layout = layout;
            }
            else {
                setColumnBorder(constants_js_1.POS_BOTTOM);
            }
            break;
        case 'border-top':
            if (table) {
                props.layout = item.layout || props.layout || {};
                const { layout, lineWidth: hLineWidth, lineColor: hLineColor } = colorLineLayout(props, 'h');
                layout.hLineWidth = (i, node, columnIndex) => (i === 0) ? width : hLineWidth(i, node, columnIndex);
                layout.hLineColor = (i, node, columnIndex) => (i === 0) ? color : hLineColor(i, node, columnIndex);
                props.layout = layout;
            }
            else {
                setColumnBorder(constants_js_1.POS_TOP);
            }
            break;
        case 'border-right':
            if (table) {
                props.layout = item.layout || props.layout || {};
                const { layout, lineWidth: vLineWidth, lineColor: vLineColor } = colorLineLayout(props, 'v');
                layout.vLineWidth = (i, node, columnIndex) => (node.table.body.length === 1 ? i === node.table.body.length : i % node.table.body.length !== 0) ? width : vLineWidth(i, node, columnIndex);
                layout.vLineColor = (i, node, columnIndex) => (node.table.body.length === 1 ? i === node.table.body.length : i % node.table.body.length !== 0) ? color : vLineColor(i, node, columnIndex);
                props.layout = layout;
            }
            else {
                setColumnBorder(constants_js_1.POS_RIGHT);
            }
            break;
        case 'border-left':
            if (table) {
                props.layout = item.layout || props.layout || {};
                const { layout, lineWidth: vLineWidth, lineColor: vLineColor } = colorLineLayout(props, 'v');
                layout.vLineWidth = (i, node, columnIndex) => (node.table.body.length === 1 ? i === 0 : i % node.table.body.length === 0) ? width : vLineWidth(i, node, columnIndex);
                layout.vLineColor = (i, node, columnIndex) => (node.table.body.length === 1 ? i === 0 : i % node.table.body.length === 0) ? color : vLineColor(i, node, columnIndex);
                props.layout = layout;
            }
            else {
                setColumnBorder(constants_js_1.POS_LEFT);
            }
            break;
    }
};
exports.computeBorder = computeBorder;
