"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.computePadding = void 0;
const constants_js_1 = require("../constants.js");
const type_guards_js_1 = require("../utils/type-guards.js");
const unit_js_1 = require("../utils/unit.js");
const paddings = {
    'padding-top': constants_js_1.POS_TOP,
    'padding-left': constants_js_1.POS_LEFT,
    'padding-right': constants_js_1.POS_RIGHT,
    'padding-bottom': constants_js_1.POS_BOTTOM
};
const computePadding = (item, props, directive, input) => {
    const index = typeof paddings[directive] !== 'undefined' ? paddings[directive] : -1;
    const value = (0, unit_js_1.toUnit)(input);
    if ((0, type_guards_js_1.isTable)(item)) {
        props.layout = item.layout || props.layout || {};
        if (typeof props.layout === 'string') {
            props.layout = {};
        }
        switch (index) {
            case constants_js_1.POS_LEFT:
                props.layout.paddingLeft = () => value;
                break;
            case constants_js_1.POS_TOP:
                props.layout.paddingTop = () => value;
                break;
            case constants_js_1.POS_RIGHT:
                props.layout.paddingRight = () => value;
                break;
            case constants_js_1.POS_BOTTOM:
                props.layout.paddingBottom = () => value;
                break;
            default:
                throw new Error('Unsupported index for "' + directive + '": ' + index);
        }
    }
    else {
        const padding = props[constants_js_1.META][constants_js_1.PADDING] || item[constants_js_1.META]?.[constants_js_1.PADDING] || [0, 0, 0, 0];
        padding[index] = value;
        props[constants_js_1.META][constants_js_1.PADDING] = [...padding];
        if (!(0, type_guards_js_1.isTdOrTh)(item)) {
            const margin = props.margin ?? [0, 0, 0, 0];
            props.margin = margin;
            const marginValue = margin[index];
            props.margin[index] = value + marginValue;
        }
    }
};
exports.computePadding = computePadding;
