"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collapseWhitespace = exports.collapseMargin = void 0;
const constants_js_1 = require("../constants.js");
const type_guards_js_1 = require("./type-guards.js");
const collapseMargin = (item, prevItem) => {
    if ((0, type_guards_js_1.isCollapsable)(item) && (0, type_guards_js_1.isCollapsable)(prevItem)) {
        const prevMargin = prevItem[constants_js_1.META]?.[constants_js_1.MARGIN] || [0, 0, 0, 0];
        prevItem[constants_js_1.META] = { ...(prevItem[constants_js_1.META] || {}), [constants_js_1.MARGIN]: prevMargin };
        prevItem.margin[constants_js_1.POS_BOTTOM] = prevItem[constants_js_1.META]?.[constants_js_1.PADDING]?.[constants_js_1.POS_BOTTOM] || 0;
        const itemMargin = item[constants_js_1.META]?.[constants_js_1.MARGIN] || [0, 0, 0, 0];
        const marginTop = Math.max(itemMargin[constants_js_1.POS_TOP], prevMargin[constants_js_1.POS_BOTTOM]);
        itemMargin[constants_js_1.POS_TOP] = marginTop;
        prevMargin[constants_js_1.POS_BOTTOM] = 0;
        item[constants_js_1.META] = { ...(item[constants_js_1.META] || {}), [constants_js_1.MARGIN]: itemMargin };
        item.margin[constants_js_1.POS_TOP] = marginTop + (item[constants_js_1.META]?.[constants_js_1.PADDING]?.[constants_js_1.POS_TOP] || 0);
    }
};
exports.collapseMargin = collapseMargin;
const findLastDeep = (ta) => {
    const last = ta.text.at(-1);
    if ((0, type_guards_js_1.isTextArray)(last)) {
        return findLastDeep(last);
    }
    return last;
};
const findFirstArrayDeep = (ta) => {
    const first = ta.text.at(0);
    if ((0, type_guards_js_1.isTextArray)(first)) {
        return findFirstArrayDeep(first);
    }
    return ta.text;
};
const collapseWhitespace = (item, nextText) => {
    const prevLastText = findLastDeep(item);
    const nextFirstTextArray = findFirstArrayDeep(nextText);
    if (prevLastText && prevLastText[constants_js_1.META]?.[constants_js_1.IS_WHITESPACE] && nextFirstTextArray?.[0]?.[constants_js_1.META]?.[constants_js_1.IS_WHITESPACE]) {
        nextFirstTextArray.shift();
    }
};
exports.collapseWhitespace = collapseWhitespace;
