"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.merge = exports.isObject = void 0;
/**
 * @description Method to check if an item is an object. Date and Function are considered
 * an object, so if you need to exclude those, please update the method accordingly.
 * @param item - The item that needs to be checked
 * @return {Boolean} Whether or not @item is an object
 */
const isObject = (item) => {
    return (item === Object(item) && !Array.isArray(item));
};
exports.isObject = isObject;
/**
 * @description Method to perform a deep merge of objects
 * @param {Object} target - The targeted object that needs to be merged with the supplied @sources
 * @param {Array<Object>} sources - The source(s) that will be used to update the @target object
 * @return {Object} The final merged object
 */
const merge = (target, ...sources) => {
    // return the target if no sources passed
    if (!sources.length) {
        return target;
    }
    const result = target;
    if ((0, exports.isObject)(result)) {
        for (let i = 0; i < sources.length; i += 1) {
            if ((0, exports.isObject)(sources[i])) {
                const elm = sources[i];
                Object.keys(elm).forEach(key => {
                    if ((0, exports.isObject)(elm[key])) {
                        if (!result[key] || !(0, exports.isObject)(result[key])) {
                            result[key] = {};
                        }
                        (0, exports.merge)(result[key], elm[key]);
                    }
                    else {
                        result[key] = elm[key];
                    }
                });
            }
        }
    }
    return result;
};
exports.merge = merge;
