"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.expandValueToUnits = exports.toUnitsOrValues = exports.toUnitOrValue = exports.getUnitOrValue = exports.toUnit = void 0;
const toUnit = (value, rootPt = 12) => {
    // if it's just a number, then return it
    if (typeof value === 'number') {
        return isFinite(value) ? value : 0;
    }
    const val = Number(parseFloat(value));
    if (isNaN(val)) {
        return 0;
    }
    const match = ('' + value).trim().match(/(pt|px|r?em|cm)$/);
    if (!match) {
        return val;
    }
    switch (match[1]) {
        case 'em':
        case 'rem':
            return val * rootPt;
        case 'px':
            // 1px = 0.75 Point
            return Number((val * 0.75).toFixed(2));
        case 'cm':
            return Number((val * 28.34).toFixed(2));
        case 'mm':
            return Number((val * 10 * 28.34).toFixed(2));
        default:
            return val;
    }
};
exports.toUnit = toUnit;
const getUnitOrValue = (value) => typeof value === 'string' && (value.indexOf('%') > -1 || value.indexOf('auto') > -1)
    ? value
    : (0, exports.toUnit)(value);
exports.getUnitOrValue = getUnitOrValue;
const toUnitOrValue = (value) => (0, exports.getUnitOrValue)(value);
exports.toUnitOrValue = toUnitOrValue;
const toUnitsOrValues = (value) => value.map(v => (0, exports.getUnitOrValue)(v));
exports.toUnitsOrValues = toUnitsOrValues;
const expandValueToUnits = (value) => {
    const values = (0, exports.toUnitsOrValues)(value.split(' ')
        .map(v => v.trim())
        .filter(v => v));
    if (values === null || !Array.isArray(values)) {
        return null;
    }
    // values[0] = top
    // values[1] = right
    // values[2] = bottom
    // values[3] = left
    // pdfmake use left, top, right, bottom, ||  [horizontal, vertical]
    // css use top, right, bottom, left
    if (values.length === 1 && values[0] !== null) {
        return [values[0], values[0], values[0], values[0]];
    }
    else if (values.length === 2) {
        // topbottom leftright
        return [values[1], values[0], values[1], values[0]];
    }
    else if (values.length === 3) {
        // top bottom leftright
        return [values[2], values[0], values[2], values[1]];
    }
    else if (values.length === 4) {
        return [values[3], values[0], values[1], values[2]];
    }
    return null;
};
exports.expandValueToUnits = expandValueToUnits;
